@@ -1,21 +1,19 @@
 #!/usr/bin/perl -w
 
 if ($#ARGV == 0) {
-  $file = $ARGV[0];
+    $file = $ARGV[0];
 } else {
-  $file = "./gri.archive";
+    $file = "./gri.archive";
 }
 open(IN, $file) or die "Cannot open $file";
 
 print "<html>
-
-<body bgcolor=\"#ffffff\" link=\"#993333\" vlink=\"#333399 alink=\"#cc3333\">
-
 <head>
 <title>Gri newsgroup archive</title>
 </head>
+<body bgcolor=\"#ffffff\" link=\"#993333\" vlink=\"#333399\">
 
-<h1>20.3: Archive of messages sent to the gri newsgroup</h2>
+<h2>12.8: Archive of messages sent to the gri newsgroup</h2>
 
 ";
 
@@ -28,16 +26,18 @@ $line = 0;
 while(<IN>) {
     die if $line++>100000;
     chop;
+    s/&/&amp;/g;  s/</&lt;/g;    s/>/&gt;/g;    # Fix <>.
     if (/^From /) {
 	#print "Got From ...\n";
 	while(<IN>) {
+	    s/&/&amp;/g;  s/</&lt;/g;    s/>/&gt;/g;    # Fix <>.
 	    chop;
 	    #print "Got new line '$_'\n";
 	    if (/Date:\s(.*)/)    { $date    = $1; }
 	    if (/Subject:\s(.*)/) { $subject = $1; }
 	    if (/From:\s(.*)/)    { $from    = $1; }
 	    if (/X-Status:/) {
-		print "<h2>20.3.$count. $subject</h2>\n";
+		print "<h2>12.8.$count. $subject</h2>\n";
 		print "<i>On $date, $from wrote </i>...<br>\n";
 		$date = "(unknown date)";
 		$subject = "(unknown subject)";
