@@ -1,6 +1,6 @@
 /*
     Gri - A language for scientific graphics programming
-    Copyright (C) 2008 Daniel Kelley
+    Copyright (C) 2010 Daniel Kelley
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -386,7 +386,7 @@ convert_col_to_gridCmd()
 				_legit_xy(i, j) = false;
 	matrix_limits(&_f_min, &_f_max);
 	if (get_flag("jinyu1")) {
-		char *name = "tmp.dat";
+		const char *name = "tmp.dat";
 		printf("Flag 'jinyu1' set, so writing x, y, z, zpredicted to file '%s'\n",
 		       name);
 		FILE *tmp = fopen (name, "w");
@@ -402,6 +402,7 @@ convert_col_to_gridCmd()
 				fprintf(tmp, "%f %f %f %f\n", 
 					_colX[i], _colY[i], _colZ[i], gr_currentmissingvalue());
 			}
+			fclose(tmp);
 		}
 	}
 	return found;
@@ -1076,10 +1077,10 @@ X data must be ordered and distinct.\n\
 								     factor * 3.) + c * (zt2 * 3. - 1.));
 		coef[(*l << 2) + 3] = (d * alpha * onemzt + c * zeta) * 6.;
 		coef[(*l << 2) + 4] = (c - d * alpha) * 6. / s[i + s_dim1];
-		coef[(*l - 1 << 2) + 4] = coef[(*l << 2) + 4] - d * 6. * ((
-			double)1. - alpha) / (del * zt2);
-		coef[(*l - 1 << 2) + 2] = coef[(*l << 2) + 2] - del * (coef[(*l << 2) 
-									   + 3] - del / 2. * coef[(*l - 1 << 2) + 4]);
+		coef[((*l - 1) << 2) + 4] = coef[(*l << 2) + 4] - d * 6. * ((
+									     double)1. - alpha) / (del * zt2);
+		coef[((*l - 1) << 2) + 2] = coef[(*l << 2) + 2] - del * (coef[(*l << 2) 
+									      + 3] - del / 2. * coef[((*l - 1) << 2) + 4]);
 		goto L68;
 	L62:
 		coef[(*l << 2) + 2] = divdif - s[i + s_dim1] * (s[i + (s_dim1 << 2)] *
@@ -1105,11 +1106,11 @@ X data must be ordered and distinct.\n\
 		++(*l);
 		// Computing 3rd power
 		r_1 = onemzt, r_2 = r_1;
-		coef[(*l << 2) + 4] = coef[(*l - 1 << 2) + 4] + (1. - alpha) * 
+		coef[(*l << 2) + 4] = coef[((*l - 1) << 2) + 4] + (1. - alpha) * 
 			6. * c / (s[i + s_dim1] * (r_2 * (r_1 * r_1)));
-		coef[(*l << 2) + 3] = coef[(*l - 1 << 2) + 3] + del * coef[(*l - 1 << 2) + 4];
-		coef[(*l << 2) + 2] = coef[(*l - 1 << 2) + 2] + del * (coef[(*l - 1 << 2) + 3] + del / 2. * coef[(*l - 1 << 2) + 4]);
-		coef[(*l << 2) + 1] = coef[(*l - 1 << 2) + 1] + del * (coef[(*l - 1 << 2) + 2] + del / 2. * (coef[(*l - 1 << 2) + 3] + del / 3. * coef[(*l - 1 << 2) + 4]));
+		coef[(*l << 2) + 3] = coef[((*l - 1) << 2) + 3] + del * coef[((*l - 1) << 2) + 4];
+		coef[(*l << 2) + 2] = coef[((*l - 1) << 2) + 2] + del * (coef[((*l - 1) << 2) + 3] + del / 2. * coef[((*l - 1) << 2) + 4]);
+		coef[(*l << 2) + 1] = coef[((*l - 1) << 2) + 1] + del * (coef[((*l - 1) << 2) + 2] + del / 2. * (coef[((*l - 1) << 2) + 3] + del / 3. * coef[((*l - 1) << 2) + 4]));
 		goto L68;
 	L65:
 		coef[(*l << 2) + 2] = divdif;
@@ -1374,12 +1375,13 @@ convert_grid_to_columnsCmd(void)
 bool
 convert_grid_to_imageCmd()
 {
-	//printf("%s:%d ENTER convert_grid_to_imageCmd()\n", __FILE__,__LINE__);
-	int             i, ii, j, jj, val;
+        //printf("\n%s:%d ENTER convert_grid_to_imageCmd()\n", __FILE__,__LINE__);
+        int             i, ii, j, jj, val;
 	int             width, height;
 	int             clipped = 0, masked = 0;
 	double          value, xx, dxx, yy, dyy;
 	double          scale;
+        bool            directly = false;
 	if (!_grid_exists) {
 		err("Cannot `convert grid to image' since no grid data exist yet");
 		return false;
@@ -1405,10 +1407,17 @@ convert_grid_to_imageCmd()
 	// See if the image size was given.
 	width = IMAGE_SIZE_WHEN_CONVERTING;
 	height = IMAGE_SIZE_WHEN_CONVERTING;
-	if (2 == get_cmd_values(_word, _nword, "size", 2, _dstack)) {
+        // convert grid to image directly
+        if (word_is(4, "directly")) {
+                width = (int)_num_xmatrix_data;
+                height = (int) _num_ymatrix_data;
+                directly = true;
+                //printf("DEBUG: using 'directly' method. %d by %d\n", width, height);
+        } else if (2 == get_cmd_values(_word, _nword, "size", 2, _dstack)) {
 		width = (int) (0.5 + _dstack[0]);
 		height = (int) (0.5 + _dstack[1]);
-	}
+	} 
+        //printf("DEBUG image width=%d   height=%d\n", width, height);
 	// BEGIN: Code prior to 2.005
 	//    if (width % 2) {
 	//	warning("Making .width. in `convert grid to image' an even number");
@@ -1443,7 +1452,11 @@ convert_grid_to_imageCmd()
 		err("`box .xleft. .ybottom. .xright. .ytop.' needs exactly 4 parameters");
 		return false;
 	}
-	if (width < 2) { err("Can't have grid < 2 wide"); return false; }
+
+        //printf("_xmatrix[0]= %f  _ymatrix[0] = %f   _xmatrix[right] = %f   _ymatrix[top] = %f\n", 
+        //_xmatrix[0], _ymatrix[0], _xmatrix[_num_xmatrix_data - 1], _ymatrix[_num_ymatrix_data - 1]);
+
+	if (width  < 2) { err("Can't have grid < 2 wide"); return false; }
 	if (height < 2) { err("Can't have grid < 2 tall"); return false; }
 
 	//printf("%s:%d convert_grid_to_imageCmd() ABOUT TO GET STORAGE FOR %d by %d \n", __FILE__,__LINE__,width,height);
@@ -1469,39 +1482,81 @@ convert_grid_to_imageCmd()
 	}
 	bool warned = false;
 	GriTimer t;
-	for (i = 0; i < width; i++) {
-		xx = _image_llx + i * dxx;
-		for (j = 0; j < height; j++) {
-			yy = _image_lly + j * dyy;
-			if (!locate_i_j(xx, yy, &ii, &jj)
-			    || !value_i_j(ii, jj, xx, yy, &value)) {
-				*(_image.image + _image.ras_height * i + j) = (unsigned char) _imageBLANK;
-				*(_imageMask.image + _imageMask.ras_height * i + j) = 2;
-				masked++;
-			} else {
-				// Method for converting to integer follows that in
-				// value_to_image(), but done here to speed up.  Make sure to
-				// update this if value_to_image() is updated.
-				// XREF value_to_image()
-				val = (int) floor(0.5 + scale * (value - _image0));
-				if (val < 0) {
-					val = 0;
-					clipped++;
-				} else if (val > 255) {
-					val = 255;
-					clipped++;
-				}
-				*(_image.image + _image.ras_height * i + j) = (unsigned char)
-					val;
-				*(_imageMask.image + _imageMask.ras_height * i + j) = (unsigned char) 0;
-			}
-			if (!warned) {
-				double frac = (height * (1.0 + i)) /(width * height);
-				warned = warn_if_slow(&t, frac, "convert grid to image");
-			}
-		}
-	}
-	if (_chatty > 0) {
+        if (_chatty > 1)
+                printf("width=%d height=%d _image.ras_height=%d, _image.ras_width=%d\n", width,height,_image.ras_height,_image.ras_width);
+        if (directly) {
+                for (i = 0; i < width; i++) {
+                        xx = _image_llx + i * dxx + 0.5 * dxx;
+                        for (j = 0; j < height; j++) {
+                                yy = _image_lly + j * dyy + 0.5 * dyy;
+                                if (locate_i_j(xx, yy, &ii, &jj) && _legit_xy(ii, jj)) {
+                                        val = (int) floor(0.5 + scale * (_f_xy(ii, jj) - _image0));
+                                        if (val < 0) {
+                                                val = 0;
+                                                clipped++;
+                                        } else if (val > 255) {
+                                                val = 255;
+                                                clipped++;
+                                        }
+                                        *(_image.image + _image.ras_height * i + j) = (unsigned char) val;
+                                        *(_imageMask.image + _imageMask.ras_height * i + j) = (unsigned char) 0;
+                                } else {
+                                        *(_image.image + _image.ras_height * i + j) = (unsigned char) _imageBLANK;
+                                        *(_imageMask.image + _imageMask.ras_height * i + j) = 2;
+                                        masked++;
+                                }
+                                if (!warned) {
+                                        double frac = (height * (1.0 + i)) /(width * height);
+                                        warned = warn_if_slow(&t, frac, "convert grid to image");
+                                }
+                        }
+                }
+        } else {
+                for (i = 0; i < width; i++) {
+                        xx = _image_llx + i * dxx;
+                        for (j = 0; j < height; j++) {
+                                yy = _image_lly + j * dyy;
+                                if (!locate_i_j(xx, yy, &ii, &jj)
+                                    || !value_i_j(ii, jj, xx, yy, &value)) {
+                                        *(_image.image + _image.ras_height * i + j) = (unsigned char) _imageBLANK;
+                                        *(_imageMask.image + _imageMask.ras_height * i + j) = 2;
+                                        masked++;
+                                } else {
+                                        // Method for converting to integer follows that in
+                                        // value_to_image(), but done here to speed up.  Make sure to
+                                        // update this if value_to_image() is updated.
+                                        // XREF value_to_image()
+                                        val = (int) floor(0.5 + scale * (value - _image0));
+                                        if (val < 0) {
+                                                val = 0;
+                                                clipped++;
+                                        } else if (val > 255) {
+                                                val = 255;
+                                                clipped++;
+                                        }
+                                        *(_image.image + _image.ras_height * i + j) = (unsigned char) val;
+                                        *(_imageMask.image + _imageMask.ras_height * i + j) = (unsigned char) 0;
+                                }
+                                if (!warned) {
+                                        double frac = (height * (1.0 + i)) /(width * height);
+                                        warned = warn_if_slow(&t, frac, "convert grid to image");
+                                }
+                        }
+                }
+        }
+        if (_chatty > 2) {
+                printf("IMAGE:\n"); // NB. the image is flipped in y.
+                for (j = 0; j < height; j++) {
+                        for (i = 0; i < width; i++) {
+                                if (*(_imageMask.image + _imageMask.ras_height * i + j) == 2)
+                                        printf(".");
+                                else
+                                        printf("*");
+                        }
+                        printf("\n");
+                }
+        }
+	if (_chatty > 1) {
 		if (clipped || masked) {
 			sprintf(_grTempString, "\
 `convert grid to image':\n  Clipped %d (%.3f%%) pixels and masked %d (%.3f%%) pixels)\n",
@@ -1595,8 +1650,8 @@ convert_image_to_gridCmd()
 bool
 value_i_j(unsigned int ii, unsigned int jj, double xx, double yy, double *value)
 {
-	double          Dx, Dy;	// width/height of domain with point
-	double          f0, f1, f2, f3;
+	double Dx, Dy;          // width/height of domain with point
+	double f0, f1, f2, f3;
 	double dx;			// x - x_to_left
 	double dy;			// y - y_below
 	// Fiddle with dx,dy,Dx,Dy, to avoid looking past array
